package de.bsvrz.sys.usv.enums;

/**
 * Zuordnung der Zustnde des Attributs EingangsFrequenZustand der
 * Attributgruppe USV-Status
 * 
 * @author ObertM
 * @author Christian Hsel, BitCtrl Systems GmbH
 * 
 */
public enum EingangsfrequenzZustand {

	/**
	 * Der {@link EingangsfrequenzZustand} "Nicht ermittelbar" (-1).
	 */
	na("nicht ermittelbar", -1),

	/**
	 * Der {@link EingangsfrequenzZustand} "in Ordnung" (0).
	 */
	normal("in Ordnung", 0),

	/**
	 * Der {@link EingangsfrequenzZustand} "Hoch" (1).
	 */
	hoch("Hoch", 1),

	/**
	 * Der {@link EingangsfrequenzZustand} "Niedrig" (2).
	 */
	niedrig("Niedrig", 2);

	private final String m_text;
	private final int m_value;

	/**
	 * Privater Konstruktor zu Erzeugung eines {@link EingangsfrequenzZustand}es.
	 * 
	 * @param text
	 *            der Name des {@link EingangsfrequenzZustand}es.
	 * @param value
	 *            die Ordinalzahl des {@link EingangsfrequenzZustand}es.
	 */
	private EingangsfrequenzZustand(final String text, final int value) {
		m_text = text;
		m_value = value;
	}

	/**
	 * Liefert den Namen des {@link EingangsfrequenzZustand}es.
	 * 
	 * @return der Name des {@link EingangsfrequenzZustand}es.
	 */
	public String getText() {
		return m_text;
	}

	/**
	 * Liefert die Ordinalzahl des {@link EingangsfrequenzZustand}es.
	 * 
	 * @return die Ordinalzahl des {@link EingangsfrequenzZustand}es.
	 */
	public int getValue() {
		return m_value;
	}
}