package de.bsvrz.sys.usv.enums;

/**
 * Zuordnung der Zustnde des Attributs WechselSpannung der Attributgruppe
 * USV-Status
 * 
 * @author ObertM
 * @author Christian Hsel, BitCtrl Systems GmbH
 * 
 */
public enum WechselSpannung {

	/**
	 * Die {@link WechselSpannung} "Nicht ermittelbar" (-1).
	 */
	na("nicht ermittelbar", -1),

	/**
	 * Die {@link WechselSpannung} "in Ordnung" (0).
	 */
	normal("in Ordnung", 0),

	/**
	 * Die {@link WechselSpannung} "berspannung" (1).
	 */
	Ueberspannung("berspannung", 1),

	/**
	 * Die {@link WechselSpannung} "Unterspannung" (2).
	 */
	Unterspannung("Unterspannung", 2);

	private final String m_text;
	private final int m_value;

	/**
	 * Privater Konstruktor zur Erzeugung eines {@link WechselSpannung}s
	 * Zustandes.
	 * 
	 * @param text
	 *            der Names des {@link WechselSpannung}s Zustandes.
	 * @param value
	 *            die Ordinalzahl des {@link WechselSpannung}s Zustandes.
	 */
	private WechselSpannung(final String text, final int value) {
		m_text = text;
		m_value = value;
	}

	/**
	 * Liefert der Namen des {@link WechselSpannung}s Zustandes.
	 * 
	 * @return der Name des {@link WechselSpannung}s Zustandes.
	 */
	public String getText() {
		return m_text;
	}

	/**
	 * Liefert die Ordinalzahl des {@link WechselSpannung}s Zustandes.
	 * 
	 * @return die Ordinalzahl des {@link WechselSpannung}s Zustandes.
	 */
	public int getValue() {
		return m_value;
	}
}
