package de.bsvrz.sys.usv.enums;

/**
 * Zuordnung der Zustnde der Attribute ZustandPhase1 .. ZustandPhase3 der
 * Attributgruppe USV-Status
 * 
 * @author ObertM
 * @author Christian Hsel, BitCtrl Systems GmbH
 * 
 */
public enum ZustandPhase {

	/**
	 * Der {@link ZustandPhase} "Nicht ermittelbar" (-1).
	 */
	na("nicht ermittelbar", -1),

	/**
	 * Der {@link ZustandPhase} "in Ordnung" (0).
	 */
	normal("in Ordnung", 0),

	/**
	 * Der {@link ZustandPhase} "Strung" (1).
	 */
	Stoerung("Strung", 1);

	private final String m_text;
	private final int m_value;

	/**
	 * Privater Konstruktor zur Erzeugung eines Phasenzustandes (
	 * {@link ZustandPhase} ).
	 * 
	 * @param text
	 *            der Name des {@link ZustandPhase}.
	 * @param value
	 *            die Ordinalzahl des {@link ZustandPhase}.
	 */
	private ZustandPhase(final String text, final int value) {
		m_text = text;
		m_value = value;
	}

	/**
	 * Liefert den Namen eines {@link ZustandPhase}.
	 * 
	 * @return der Name des {@link ZustandPhase}.
	 */
	public String getText() {
		return m_text;
	}

	/**
	 * Liefert die Ordinalzahl eines {@link ZustandPhase}.
	 * 
	 * @return die Ordinalzahl des {@link ZustandPhase}.
	 */
	public int getValue() {
		return m_value;
	}
}
