/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.usv.usv;

import de.bsvrz.sys.usv.enums.EingangsfrequenzZustand;
import de.bsvrz.sys.usv.enums.KommunikationsZustand;
import de.bsvrz.sys.usv.enums.USVZustand;
import de.bsvrz.sys.usv.enums.WechselSpannung;
import de.bsvrz.sys.usv.enums.ZustandNotstrom;
import de.bsvrz.sys.usv.enums.ZustandPhase;
import de.bsvrz.sys.usv.wrapper.USVAllgemein;
import de.bsvrz.sys.usv.wrapper.USVWrapper;

/**
 * Klasse realisiert eine "Dummy-USV". Sie implementiert daher das Interface
 * USVAllgemein. Alle get-Methoden werden in dieser Klasse mit dem Wert -1 fr
 * "nicht ermittelbar" beantwortet, alle set-Methoden mit dem Wert -1 fr "nicht
 * implementiert". Fr die Realisierung der Anbindung einer konkreten USV gibt
 * es daher folgende Mglichkeiten: 1. Erstellen einer Klasse die das Interface
 * USV-Allgemein implementiert, 2. Ableiten der Klasse DefaultUSV und
 * berschreiben der tatschlich in der realen USV bentigten Klassen.
 * 
 * @author ObertM Geprft 30.9.2008 Hilpp
 */

public class DefaultUSV implements USVAllgemein {
	/*
	 * (non-Javadoc)
	 * 
	 * @see de.bsvrz.sys.usv.wrapper.USVAllgemein#getUsvZustand()
	 */
	public USVZustand getUsvZustand() {
		return USVZustand.na;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.bsvrz.sys.usv.wrapper.USVAllgemein#getBatterieZustand()
	 */
	public int getBatterieZustand() {
		return -1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.bsvrz.sys.usv.wrapper.USVAllgemein#getEingangsFrequenzZustand()
	 */
	public EingangsfrequenzZustand getEingangsFrequenzZustand() {
		return EingangsfrequenzZustand.na;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.bsvrz.sys.usv.wrapper.USVAllgemein#getKommunikationsZustand()
	 */
	public KommunikationsZustand getKommunikationsZustand() {
		return KommunikationsZustand.na;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.bsvrz.sys.usv.wrapper.USVAllgemein#getWechselSpannung()
	 */
	public WechselSpannung getWechselSpannung() {
		return WechselSpannung.na;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.bsvrz.sys.usv.wrapper.USVAllgemein#getZustandNotstrom()
	 */
	public ZustandNotstrom getZustandNotstrom() {
		return ZustandNotstrom.na;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.bsvrz.sys.usv.wrapper.USVAllgemein#getZustandPhase1()
	 */
	public ZustandPhase getZustandPhase1() {
		return ZustandPhase.na;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.bsvrz.sys.usv.wrapper.USVAllgemein#getZustandPhase2()
	 */
	public ZustandPhase getZustandPhase2() {
		return ZustandPhase.na;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.bsvrz.sys.usv.wrapper.USVAllgemein#getZustandPhase3()
	 */
	public ZustandPhase getZustandPhase3() {
		return ZustandPhase.na;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.bsvrz.sys.usv.wrapper.USVAllgemein#setNotstrom(boolean)
	 */
	public int setNotstrom(final boolean befehl) {
		return -1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.bsvrz.sys.usv.wrapper.USVAllgemein#setUsvBetrieb(boolean)
	 */
	public int setUsvBetrieb(final boolean befehl) {
		return -1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.bsvrz.sys.usv.wrapper.USVAllgemein#setInstance(de.bsvrz.sys.usv.wrapper.USVWrapper)
	 */
	public void setInstance(final USVWrapper instance) {
		// nothing
	}

	public int getBatterieSpannung() {
		return -1;
	}

	public int getBatterieTemperatur() {
		return -1;
	}

	public int getBetriebsdauerBatterie() {
		return -1;
	}

	public int getUSVRestlaufzeit() {
		return -1;
	}
}
