/*
 * <Kurze Beschreibung der Anwendung, Bibliothek oder des Java-Packages (hchstens eine Zeile)>
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.sys.usv.wrapper;

import java.util.ArrayList;
import java.util.NoSuchElementException;
import java.util.Stack;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Parser, der eine Schwellwertbedingung zusammen mit einem {@link Data}
 * einliet und prft. Die Vorlage dazu stammt von www.java-forum.org. Der
 * Parser enthlt keine Syntaxprfung kann aber mit Klammerausdrcken umgehen.
 * 
 * @author BitCtrl Systems GmbH, Christian Hsel
 * @version $Id$
 */
public class SchwellwertParser {

	private static ArrayList<Operator> operators = new ArrayList<Operator>();
	static {

		operators.add(new Operator("&", 2, 2) {
			@Override
			boolean calculate(final Object[] operands) {
				boolean result = false;
				if (operands[0] instanceof Boolean
						&& operands[1] instanceof Boolean) {
					result = (Boolean) operands[0] && (Boolean) operands[1];
				}

				return result;
			}
		});
		operators.add(new Operator("|", 2, 1) {
			@Override
			boolean calculate(final Object[] operands) {
				boolean result = false;
				if (operands[0] instanceof Boolean
						&& operands[1] instanceof Boolean) {
					result = (Boolean) operands[0] || (Boolean) operands[1];
				}

				return result;
			}
		});

		operators.add(new Operator("groesser", 2, 4) {
			@Override
			boolean calculate(final Object[] operands) {
				boolean result = false;
				if (operands[0] instanceof Comparable<?>
						&& operands[1] instanceof Comparable<?>
						&& operands[0] instanceof Number
						&& operands[1] instanceof Number) {
					if ((((Comparable<Object>) operands[0])
							.compareTo(operands[1])) > 0) {
						result = true;
					}
				}
				return result;
			}
		});

		operators.add(new Operator("kleiner", 2, 4) {
			@Override
			boolean calculate(final Object[] operands) {
				boolean result = false;
				if (operands[0] instanceof Comparable<?>
						&& operands[1] instanceof Comparable<?>
						&& operands[0] instanceof Number
						&& operands[1] instanceof Number) {
					if ((((Comparable<Object>) operands[0])
							.compareTo(operands[1])) < 0) {
						result = true;
					}
				}
				return result;
			}
		});

		operators.add(new Operator("gleich", 2, 4) {
			@Override
			boolean calculate(final Object[] operands) {
				boolean result = false;
				if (operands[0] instanceof Comparable<?>
						&& operands[1] instanceof Comparable<?>
						&& operands[0] instanceof Number
						&& operands[1] instanceof Number) {
					if ((((Comparable<Object>) operands[0])
							.compareTo(operands[1])) == 0) {
						result = true;
					}
				} else {
					result = operands[0].equals(operands[1]);
				}
				return result;
			}
		});
	}

	/**
	 * Wertet die Schwellwertbedingung mit den aktuellen Zustandsdaten aus.
	 * 
	 * @param schwellwertBedingung
	 *            die Schwellwertbedingung.
	 * @param data
	 *            die aktuellen Zustandsdaten.
	 * @return <code>true</code>, wenn die Schwellwertbedingung erfllt ist,
	 *         sonst <code>false</code>.
	 */
	public static boolean auswertung(final String schwellwertBedingung,
			final Data data) {
		ArrayList<Object> expr = tokenize(schwellwertBedingung);
		Debug.getLogger().info("Schwellwertbedingung In-Fix Notation: " + expr);

		expr = convertToPostOrder(expr);
		Debug.getLogger().info(
				"Schwellwertbedingung Post-Fix Notation: " + expr);

		return eval(expr, data);
	}

	/**
	 * Wertet die Schwellwertbedingung mit den aktuellen Zustandsdaten aus.<br>
	 * <b>Achtung:</b> Das im Moment noch nicht existierende Attribut
	 * <code>BatterieRestlaufzeit</code> wird hier mit ausgewertet. Eigentlich
	 * (nach Aenderung des Datenkatalogs) wird nur die <code>auswertung</code>-Methode
	 * oben benoetigt.
	 * 
	 * @param schwellwertBedingung
	 *            die Schwellwertbedingung.
	 * @param data
	 *            die aktuellen Zustandsdaten.
	 * @param batterieRestLaufZeitInMin
	 *            die aktuelle Restlaufzeit der USV. Dieser Wert ist im Moment
	 *            noch nicht in <code>data</code> enthalten, wird aber
	 *            nachgepflegt (von Dambach).
	 * @return <code>true</code>, wenn die Schwellwertbedingung erfllt ist,
	 *         sonst <code>false</code>.
	 */
	@Deprecated
	public static boolean auswertungMitRestLaufZeit(
			final String schwellwertBedingung, final Data data,
			final long batterieRestLaufZeitInMin) {
		ArrayList<Object> expr = tokenize(schwellwertBedingung);
		Debug.getLogger().info("Schwellwertbedingung In-Fix Notation: " + expr);

		expr = convertToPostOrder(expr);
		Debug.getLogger().info(
				"Schwellwertbedingung Post-Fix Notation: " + expr);

		// TODO: wieder alte Methode benutzen (nach
		// Datenkatalogaenderung)
		// return eval(expr, data);

		return evalMitRestLaufZeit(expr, data, batterieRestLaufZeitInMin);
	}

	/**
	 * Zerlegt die Schwellwertbedingung in einzelne Tokens.
	 * 
	 * @param text
	 *            die Schwellwertbedingung.
	 * @return eine Liste mit allen Bestandteilen (Tokens) der
	 *         Schwellwertbedingung.
	 */
	private static ArrayList<Object> tokenize(final String str) {

		final ArrayList<Object> output = new ArrayList<Object>();

		for (int i = 0; i < str.length(); i++) {
			final char c = str.charAt(i);

			// Nicht Whitespaces sondern ':' sind unsere Trenner
			if (c == ':') {
				continue;
			}
			int end = i + 1;
			if (Character.isLetterOrDigit(c) || Character.isWhitespace(c)) {
				while (end < str.length()
						&& (Character.isLetterOrDigit(str.charAt(end)) || Character
								.isWhitespace(str.charAt(end)))) {
					end++;
				}
			}
			final String word = str.substring(i, end);
			int index = -1;
			for (final Operator op : operators) {
				if (op.equals(word)) {
					index = operators.indexOf(op);
				}
			}
			if (index >= 0) {
				output.add(operators.get(index));
			} else {

				output.add(word);
			}
			i = end - 1;
		}
		return output;
	}

	/**
	 * Konvertiert die Reihenfolge der Schwellwert-Tokens von In-Order nach
	 * Post-Order. D.h. Operator,Verknpfung,Operator -->
	 * Operator,Operator,Verknpfung.
	 * 
	 * @param expr
	 *            Liste der Schwellwert-Tokens in In-Order.
	 * @return Liste der Schwellwert-Tokens in Post-Order.
	 */
	private static ArrayList<Object> convertToPostOrder(
			final ArrayList<Object> expr) {
		final ArrayList<Object> output = new ArrayList<Object>();
		final Stack<Object> stack = new Stack<Object>();

		for (final Object o : expr) {
			if (o instanceof String) {
				if (o.equals("(")) {
					stack.push(o);
				} else if (o.equals(")")) {
					while (true) {
						final Object o2 = stack.pop();
						if (o2.equals("(")) {
							break;
						}
						output.add(o2);
					}
				} else {
					output.add(o);
				}
			} else if (o instanceof Operator) {
				final Operator op = (Operator) o;
				while (!stack.empty()) {
					final Object o2 = stack.pop();
					if (o2.equals("(")
							|| ((Operator) o2).getPriority() < op.getPriority()) {
						stack.push(o2);
						break;
					}
					output.add(o2);
				}
				stack.push(o);
			} else {
				throw new IllegalArgumentException(
						"Der Schwellwertausdruck enthlt Zeichen oder Ausdrcke, die nicht zugeordnet werden knnen:"
								+ o);
			}
		}
		while (!stack.empty()) {
			output.add(stack.pop());
		}
		return output;
	}

	/**
	 * Ermittelt das Ergebnis der Schwellwertbedingung unter Bercksichtung des
	 * bergebenen {@link Data}s.
	 * 
	 * @param expr
	 *            Liste der SchwellwertBedingungen in Post-Order-Notation
	 *            (Reihenfolge).
	 * @param data
	 *            der aktuelle Datensatz.
	 * @return <code>true</code>, wenn die Schwellwertbedingung erfllt ist,
	 *         sonst <code>false</code>
	 */
	private static boolean eval(final ArrayList<Object> expr, final Data data) {
		for (int i = 0; i < expr.size(); i++) {
			if (expr.get(i) instanceof String) {

				final String variable = (String) expr.get(i);
				try {
					final Data.NumberValue numberValue = data
							.getScaledValue(variable);
					if (numberValue.getState() != null) {
						// ein richtiger Status
						for (int j = i; j < expr.size(); j++) {
							if (expr.get(j).equals(variable)) {
								expr.set(j, numberValue.getText());
							}
						}
					} else {
						// nur eine Zahl
						for (int j = i; j < expr.size(); j++) {
							if (expr.get(j).equals(variable)) {
								expr.set(j, numberValue.doubleValue());
							}
						}
					}
				} catch (final NoSuchElementException e) {
					// Es handelt sich hier um eine Zahl, oder einen String
					// gegen den verglichen wird.
					for (int j = i; j < expr.size(); j++) {
						if (expr.get(j).equals(variable)) {
							try {
								expr.set(j, Double.parseDouble(variable));
							} catch (final NumberFormatException e1) {
								expr.set(j, variable);
							}
						}
					}
				}
			}
		}
		final Stack<Object> stack = new Stack<Object>();
		for (final Object o : expr) {
			if (o instanceof Operator) {
				final Operator op = (Operator) o;
				final int nrOp = op.numberOfOperands();
				final Object[] operands = new Object[nrOp];
				for (int i = 0; i < nrOp; i++) {
					operands[nrOp - i - 1] = stack.pop();
				}
				stack.push(op.calculate(operands));
			} else {
				stack.push(o);
			}
		}

		final Object result = stack.pop();
		if (result instanceof Boolean) {
			return (Boolean) result;
		}
		return false;

	}

	/**
	 * Ermittelt das Ergebnis der Schwellwertbedingung unter Beruecksichtung des
	 * uebergebenen {@link Data}s.<br>
	 * <b>Achtung:</b> Das im Moment noch nicht existierende Attribut
	 * <code>BatterieRestlaufzeit</code> wird hier mit ausgewertet. Eigentlich
	 * (nach Aenderung des Datenkatalogs) wird nur die <code>eval</code>-Methode
	 * oben benoetigt.
	 * 
	 * @param expr
	 *            Liste der SchwellwertBedingungen in Post-Order-Notation
	 *            (Reihenfolge).
	 * @param data
	 *            der aktuelle Datensatz.
	 * @param batterieRestlaufzeitInMin
	 *            die aktuelle Restlaufzeit der USV. Dieser Wert ist im Moment
	 *            noch nicht in <code>data</code> enthalten, wird aber
	 *            nachgepflegt (von Dambach).
	 * @return <code>true</code>, wenn die Schwellwertbedingung erfuellt ist,
	 *         sonst <code>false</code>
	 */
	@Deprecated
	private static boolean evalMitRestLaufZeit(final ArrayList<Object> expr,
			final Data data, final long batterieRestlaufzeitInMin) {
		for (int i = 0; i < expr.size(); i++) {

			if (expr.get(i) instanceof String) {

				final String variable = (String) expr.get(i);

				if (variable.toLowerCase().equals(
						UsvKonstanten.batterieRestlaufzeit.toLowerCase())) {
					for (int j = i; j < expr.size(); j++) {
						if (expr.get(j).equals(variable)) {
							expr.set(j, new Double(batterieRestlaufzeitInMin));
						}
					}
					continue;
				}

				try {
					final Data.NumberValue numberValue = data
							.getScaledValue(variable);
					if (numberValue.getState() != null) {
						// ein richtiger Status
						for (int j = i; j < expr.size(); j++) {
							if (expr.get(j).equals(variable)) {
								expr.set(j, numberValue.getText());
							}
						}
					} else {
						// nur eine Zahl
						for (int j = i; j < expr.size(); j++) {
							if (expr.get(j).equals(variable)) {
								expr.set(j, numberValue.doubleValue());
							}
						}
					}
				} catch (final NoSuchElementException e) {
					// Es handelt sich hier um eine Zahl, oder einen String
					// gegen den verglichen wird.
					for (int j = i; j < expr.size(); j++) {
						if (expr.get(j).equals(variable)) {
							try {
								expr.set(j, Double.parseDouble(variable));
							} catch (final NumberFormatException e1) {
								expr.set(j, variable);
							}
						}
					}
				}
			}
		}
		final Stack<Object> stack = new Stack<Object>();
		for (final Object o : expr) {
			if (o instanceof Operator) {
				final Operator op = (Operator) o;
				final int nrOp = op.numberOfOperands();
				final Object[] operands = new Object[nrOp];
				for (int i = 0; i < nrOp; i++) {
					operands[nrOp - i - 1] = stack.pop();
				}
				stack.push(op.calculate(operands));
			} else {
				stack.push(o);
			}
		}

		final Object result = stack.pop();
		if (result instanceof Boolean) {
			return (Boolean) result;
		}
		return false;

	}
}

/**
 * Beschreibt einen Operator einer Schwellwertbedingung.
 * 
 * @author BitCtrl Systems GmbH, Christian Hsel
 * @version $Id$
 */
abstract class Operator {

	private final String op;
	private final int priority;
	private final int numberOfOperands;

	Operator(final String op, final int numberOfOperands, final int priority) {
		this.op = op;
		this.priority = priority;
		this.numberOfOperands = numberOfOperands;
	}

	int getPriority() {
		return priority;
	}

	int numberOfOperands() {
		return numberOfOperands;
	}

	/**
	 * Ermittelt das Ergebnis dieses {@link Operator} mit seinen Operanten.
	 * 
	 * @param operands
	 *            die Operanten, auf denen der {@link Operator} ausgefhrt wird.
	 * @return <code>true</code>, wenn die Operation <strong>wahr</strong>
	 *         ergibt, sonst <code>false</code>.
	 */
	abstract boolean calculate(Object[] operands);

	@Override
	public String toString() {
		return op;
	}

	@Override
	public boolean equals(final Object o) {
		if (o instanceof String) {
			return ((String) o).equalsIgnoreCase(op);
		} else if (o instanceof Character) {
			return op.length() == 1
					&& op.charAt(0) == ((Character) o).charValue();
		} else {
			return this == o;
		}
	}
}
